@@ -1,6 +1,6 @@
 """Module containing pyvista implementation of vtkRenderer."""
 
-import collections
+import collections.abc
 import logging
 from weakref import proxy
 
@@ -10,10 +10,8 @@ from vtk import vtkRenderer
 
 import pyvista
 from pyvista.utilities import wrap, check_depth_peeling
-
 from .theme import parse_color, parse_font_family, rcParams, MAX_N_COLOR_BARS
-from .tools import update_axes_label_color, create_axes_orientation_box, create_axes_marker
-
+from .tools import create_axes_orientation_box, create_axes_marker
 
 
 def scale_point(camera, point, invert=False):
@@ -43,8 +41,7 @@ def scale_point(camera, point, invert=False):
     return (scaled[0], scaled[1], scaled[2])
 
 
-
-class CameraPosition(object):
+class CameraPosition:
     """Container to hold camera location attributes."""
 
     def __init__(self, position, focal_point, viewup):
@@ -100,14 +97,19 @@ class CameraPosition(object):
         self._viewup = value
 
 
-
 class Renderer(vtkRenderer):
     """Renderer class."""
 
+    # map camera_position string to an attribute
+    CAMERA_STR_ATTR_MAP = {'xy': 'view_xy', 'xz': 'view_xz',
+                           'yz': 'view_yz', 'yx': 'view_yx',
+                           'zx': 'view_zx', 'zy': 'view_zy',
+                           'iso': 'view_isometric'}
+
     def __init__(self, parent, border=True, border_color=(1, 1, 1),
                  border_width=2.0):
         """Initialize the renderer."""
-        super(Renderer, self).__init__()
+        super().__init__()
         self._actors = {}
         self.parent = parent
         self.camera_set = False
@@ -125,7 +127,6 @@ class Renderer(vtkRenderer):
         if border:
             self.add_border(border_color, border_width)
 
-
     #### Properties ####
 
     @property
@@ -141,30 +142,33 @@ class Renderer(vtkRenderer):
         """Set camera position of all active render windows."""
         if camera_location is None:
             return
-
-        if isinstance(camera_location, str):
+        elif isinstance(camera_location, str):
             camera_location = camera_location.lower()
-            if camera_location == 'xy':
-                self.view_xy()
-            elif camera_location == 'xz':
-                self.view_xz()
-            elif camera_location == 'yz':
-                self.view_yz()
-            elif camera_location == 'yx':
-                self.view_yx()
-            elif camera_location == 'zx':
-                self.view_zx()
-            elif camera_location == 'zy':
-                self.view_zy()
-            return
+            if camera_location not in self.CAMERA_STR_ATTR_MAP:
+                err = pyvista.core.errors.InvalidCameraError
+                raise err('Invalid view direction.  '
+                          'Use one of the following:\n    %s'
+                          % ', '.join(self.CAMERA_STR_ATTR_MAP))
+            getattr(self, self.CAMERA_STR_ATTR_MAP[camera_location])()
 
-        if isinstance(camera_location[0], (int, float)):
-            return self.view_vector(camera_location)
+        elif isinstance(camera_location[0], (int, float)):
+            if len(camera_location) != 3:
+                raise pyvista.core.errors.InvalidCameraError
+            self.view_vector(camera_location)
+        else:
+            # check if a valid camera position
+            if not isinstance(camera_location, CameraPosition):
+                if not len(camera_location) == 3:
+                    raise pyvista.core.errors.InvalidCameraError
+                elif any([len(item) != 3 for item in camera_location]):
+                    raise pyvista.core.errors.InvalidCameraError
 
-        # everything is set explicitly
-        self.camera.SetPosition(scale_point(self.camera, camera_location[0], invert=False))
-        self.camera.SetFocalPoint(scale_point(self.camera, camera_location[1], invert=False))
-        self.camera.SetViewUp(camera_location[2])
+            # everything is set explicitly
+            self.camera.SetPosition(scale_point(self.camera, camera_location[0],
+                                                invert=False))
+            self.camera.SetFocalPoint(scale_point(self.camera, camera_location[1],
+                                                  invert=False))
+            self.camera.SetViewUp(camera_location[2])
 
         # reset clipping range
         self.ResetCameraClippingRange()
@@ -215,7 +219,6 @@ class Renderer(vtkRenderer):
 
         return the_bounds.tolist()
 
-
     @property
     def length(self):
         """Return the length of the diagonal of the bounding box of the scene."""
@@ -241,7 +244,6 @@ class Renderer(vtkRenderer):
         self.set_background(color)
         self.Modified()
 
-
     #### Everything else ####
 
     def enable_depth_peeling(self, number_of_peels=None, occlusion_ratio=None):
@@ -291,7 +293,6 @@ class Renderer(vtkRenderer):
         self.SetUseFXAA(False)
         self.Modified()
 
-
     def add_border(self, color=[1, 1, 1], width=2.0):
         """Add borders around the frame."""
         points = np.array([[1., 1., 0.],
@@ -324,7 +325,6 @@ class Renderer(vtkRenderer):
         self.Modified()
         return actor
 
-
     def add_actor(self, uinput, reset_camera=False, name=None, culling=False,
                   pickable=True):
         """Add an actor to render window.
@@ -395,7 +395,7 @@ class Renderer(vtkRenderer):
                 except AttributeError:  # pragma: no cover
                     pass
             else:
-                raise RuntimeError('Culling option ({}) not understood.'.format(culling))
+                raise ValueError('Culling option ({}) not understood.'.format(culling))
 
         actor.SetPickable(pickable)
 
@@ -405,7 +405,6 @@ class Renderer(vtkRenderer):
 
         return actor, actor.GetProperty()
 
-
     def add_axes_at_origin(self, x_color=None, y_color=None, z_color=None,
                            xlabel='X', ylabel='Y', zlabel='Z', line_width=2,
                            labels_off=False):
@@ -426,6 +425,51 @@ class Renderer(vtkRenderer):
         self.Modified()
         return self.marker_actor
 
+    def add_orientation_widget(self, actor, interactive=None, color=None,
+                               opacity=1.0):
+        """Use the given actor in an orientation marker widget.
+
+        Color and opacity are only valid arguments if a mesh is passed.
+
+        Parameters
+        ----------
+        actor : vtk.vtkActor or pyvista.Common
+            The mesh or actor to use as the marker.
+
+        color : string, optional
+            The color of the actor.
+
+        opacity : int or float, optional
+            Opacity of the marker.
+
+        """
+        if isinstance(actor, pyvista.Common):
+            mapper = vtk.vtkDataSetMapper()
+            mesh = actor.copy()
+            mesh.clear_arrays()
+            mapper.SetInputData(mesh)
+            actor = vtk.vtkActor()
+            actor.SetMapper(mapper)
+            prop = actor.GetProperty()
+            if color is not None:
+                prop.SetColor(parse_color(color))
+            prop.SetOpacity(opacity)
+        if hasattr(self, 'axes_widget'):
+            # Delete the old one
+            self.axes_widget.EnabledOff()
+            self.Modified()
+            del self.axes_widget
+        if interactive is None:
+            interactive = rcParams['interactive']
+        self.axes_widget = vtk.vtkOrientationMarkerWidget()
+        self.axes_widget.SetOrientationMarker(actor)
+        if hasattr(self.parent, 'iren'):
+            self.axes_widget.SetInteractor(self.parent.iren)
+            self.axes_widget.SetEnabled(1)
+            self.axes_widget.SetInteractive(interactive)
+        self.axes_widget.SetCurrentRenderer(self)
+        self.Modified()
+        return self.axes_widget
 
     def add_axes(self, interactive=None, line_width=2,
                  color=None, x_color=None, y_color=None, z_color=None,
@@ -444,13 +488,16 @@ class Renderer(vtkRenderer):
         box : bool
             Show a box orientation marker. Use ``box_args`` to adjust.
             See :any:`pyvista.create_axes_orientation_box` for details.
+
+        opacity : int or float, optional
+            The opacity of the marker.
         """
         if interactive is None:
             interactive = rcParams['interactive']
         if hasattr(self, 'axes_widget'):
-            self.axes_widget.SetInteractive(interactive)
-            update_axes_label_color(color)
-            return
+            self.axes_widget.EnabledOff()
+            self.Modified()
+            del self.axes_widget
         if box is None:
             box = rcParams['axes']['box']
         if box:
@@ -466,24 +513,16 @@ class Renderer(vtkRenderer):
                 label_color=color, line_width=line_width,
                 x_color=x_color, y_color=y_color, z_color=z_color,
                 xlabel=xlabel, ylabel=ylabel, zlabel=zlabel, labels_off=labels_off)
-        self.axes_widget = vtk.vtkOrientationMarkerWidget()
-        self.axes_widget.SetOrientationMarker(self.axes_actor)
-        if hasattr(self.parent, 'iren'):
-            self.axes_widget.SetInteractor(self.parent.iren)
-            self.axes_widget.SetEnabled(1)
-            self.axes_widget.SetInteractive(interactive)
-        self.axes_widget.SetCurrentRenderer(self)
-        self.Modified()
+        self.add_orientation_widget(self.axes_actor, interactive=interactive,
+                                    color=None)
         return self.axes_actor
 
-
     def hide_axes(self):
         """Hide the axes orientation widget."""
         if hasattr(self, 'axes_widget') and self.axes_widget.GetEnabled():
             self.axes_widget.EnabledOff()
             self.Modified()
 
-
     def show_axes(self):
         """Show the axes orientation widget."""
         if hasattr(self, 'axes_widget'):
@@ -493,7 +532,6 @@ class Renderer(vtkRenderer):
             self.add_axes()
         self.Modified()
 
-
     def show_bounds(self, mesh=None, bounds=None, show_xaxis=True,
                     show_yaxis=True, show_zaxis=True, show_xlabels=True,
                     show_ylabels=True, show_zlabels=True, italic=False,
@@ -786,7 +824,6 @@ class Renderer(vtkRenderer):
         logging.warning('`add_bounds_axes` is deprecated. Use `show_bounds` or `show_grid`.')
         return self.show_bounds(*args, **kwargs)
 
-
     def show_grid(self, **kwargs):
         """Show gridlines and axes labels.
 
@@ -886,7 +923,6 @@ class Renderer(vtkRenderer):
         self.Modified()
         return self.bounding_box_actor
 
-
     def add_floor(self, face='-z', i_resolution=10, j_resolution=10,
                   color=None, line_width=None, opacity=1.0, show_edges=False,
                   lighting=False, edge_color=None, reset_camera=None, pad=0.0,
@@ -1035,7 +1071,6 @@ class Renderer(vtkRenderer):
             self.remove_actor(self.cube_axes_actor)
             self.Modified()
 
-
     def clear(self):
         """Remove all actors and properties."""
         if self._actors:
@@ -1048,7 +1083,6 @@ class Renderer(vtkRenderer):
         self.RemoveAllViewProps()
         self.Modified()
 
-
     def set_focus(self, point):
         """Set focus to a point."""
         if isinstance(point, np.ndarray):
@@ -1076,7 +1110,6 @@ class Renderer(vtkRenderer):
         self.camera.SetViewUp(vector)
         self.Modified()
 
-
     def enable_parallel_projection(self):
         """Enable parallel projection.
 
@@ -1087,13 +1120,11 @@ class Renderer(vtkRenderer):
         self.camera.SetParallelProjection(True)
         self.Modified()
 
-
     def disable_parallel_projection(self):
         """Reset the camera to use perspective projection."""
         self.camera.SetParallelProjection(False)
         self.Modified()
 
-
     def remove_actor(self, actor, reset_camera=False, render=True):
         """Remove an actor from the Renderer.
 
@@ -1134,7 +1165,7 @@ class Renderer(vtkRenderer):
             except KeyError:
                 # If actor of that name is not present then return success
                 return False
-        if isinstance(actor, collections.Iterable):
+        if isinstance(actor, collections.abc.Iterable):
             success = False
             for a in actor:
                 rv = self.remove_actor(a, reset_camera=reset_camera)
@@ -1189,7 +1220,6 @@ class Renderer(vtkRenderer):
             self.reset_camera()
         self.Modified()
 
-
     def get_default_cam_pos(self, negative=False):
         """Return the default focal points and viewup.
 
@@ -1306,7 +1336,6 @@ class Renderer(vtkRenderer):
             vec *= -1
         return self.view_vector(vec, viewup)
 
-
     def view_zy(self, negative=False):
         """View the ZY plane."""
         vec = np.array([-1,0,0])
@@ -1349,7 +1378,6 @@ class Renderer(vtkRenderer):
         self.Modified()
         return
 
-
     def get_pick_position(self):
         """Get the pick position/area as x0, y0, x1, y1."""
         x0 = int(self.GetPickX1())
@@ -1358,7 +1386,6 @@ class Renderer(vtkRenderer):
         y1 = int(self.GetPickY2())
         return x0, y0, x1, y1
 
-
     def set_background(self, color, top=None):
         """Set the background color.
 
@@ -1411,7 +1438,7 @@ class Renderer(vtkRenderer):
             self.remove_bounding_box()
 
         self.RemoveAllViewProps()
-        self._actors = None
+        self._actors = {}
         # remove reference to parent last
         self.parent = None
         return
